#Written by Emma Tait for FEMC NY Rec Data project
#This script converts eddmap shapefiles into gdb featureclasses and add states then
#merges them into point and polygon shapefiles.
#Last Updated 21 October 2019
import arcpy
import os

folder = r'I:\WorkingProjects\NY_RecData\InvasivePlantData\EDDMapS'
path = r'I:\WorkingProjects\NY_RecData\InvasivePlantData'
stateList = os.listdir(folder)

arcpy.env.scratchWorkspace = path
arcpy.env.workspace = path+'\\scratch.gdb'
arcpy.env.overwriteOutput = True;

mergeListPts = []
mergeListPoly = []
print (stateList)
# add to scratch workspace and add state attribute field
for state in stateList:
    statepath = folder+'\\'+state+'\\'
    arcpy.env.workspace = statepath
    
    points_file = arcpy.ListFeatureClasses('points*')[0]
    polygons_file = arcpy.ListFeatureClasses('polygon*')[0]
    
    
    
    points = arcpy.CopyFeatures_management(points_file,path+'\\scratch.gdb\\'+state+'_points')
    polygons = arcpy.CopyFeatures_management(polygons_file,path+'\\scratch.gdb\\'+state+'_polygons')
    
    arcpy.env.workspace = path+'\\scratch.gdb'
    
    arcpy.AddField_management(points,'State','TEXT',50)
    arcpy.CalculateField_management(points,'State','"'+state+'"')
    arcpy.AddField_management(polygons,'State','TEXT',50)
    arcpy.CalculateField_management(polygons,'State','"'+state+'"')
    mergeListPts.append(points)
    mergeListPoly.append(polygons)
    print (state+ ' '+points_file +' complete')
    print(state+ ' '+polygons_file+' complete')

arcpy.env.workspace = path+'\\scratch.gdb'

fieldLengths = {}
for pt in mergeListPts:
    for f in arcpy.ListFields(pt):
        if f.name in fieldLengths:
            if f.length > fieldLengths[f.name]:
                fieldLengths[f.name] = f.length
        else:
            fieldLengths[f.name] = f.length
          
for field in arcpy.ListFields(mergeListPts[0]):
    print ('******************************* POINT ***************************')
    print (field.name+':  '+str(fieldLengths[field.name]))
    print (field.name+':  '+str(field.length)+', type: '+field.type)
    if field.length < fieldLengths[field.name]:
        arcpy.AlterField_management(mergeListPts[0],field.name,"","","","255")


fieldLengths = {}        
for pt in mergeListPoly:
    for f in arcpy.ListFields(pt):
        if f.name in fieldLengths:
            if f.length > fieldLengths[f.name]:
                fieldLengths[f.name] = f.length
        else:
            fieldLengths[f.name] = f.length
            
for field in arcpy.ListFields(mergeListPoly[0]):
    print ('******************************* POLYGON ***************************')
    print (field.name+':  '+str(fieldLengths[field.name]))
    print (field.name+':  '+str(field.length))
    if field.length < fieldLengths[field.name]:
        print ('field length is: '+str(field.length))
        arcpy.AlterField_management(mergeListPoly[0],field.name,"","","","255")
  
print ('merge list complete')
pointMerge = arcpy.Merge_management(mergeListPts, 'points_allStates')
print ('point merge complete')
polygonMerge = arcpy.Merge_management(mergeListPoly,'polygons_allStates')

print('script complete')

