#Written by Emma Tait for FEMC NY Rec Data project
#This script matches trail register trip data with originating towns (where people are coming from)
#Last Updated 9 October 2019

## ****** IMPORTANT NOTES ********
# run this script first (before MatchDestinations.py) 
import csv

#csv of the attribute table for the originating towns
origin = r'I:\WorkingProjects\NY_RecData\SpatialData\originTables\tblOrigins.csv'
#trail register trip data
trailRegister = r'I:\WorkingProjects\NY_RecData\trailRegisterEdited.csv'
#output file for matched trips
matchedFile = r'I:\WorkingProjects\NY_RecData\tblTrips.csv'
#output file for unmatched trips
checkFile = r'I:\WorkingProjects\NY_RecData\UnmatchedTripsOrigins.csv'
#variables 
matchedEntries = []
humanCheck = []

#get all of the trips
with open(trailRegister) as trailcsv:
    trail = csv.reader(trailcsv, delimiter=',')
    tripID = 0
    for tr in trail:
        #attributes of trips
        trState = tr[9]
        trTown = tr[11]
        trCounty = tr[13]
        ogTown = tr[10]
        mgmtUnit = tr[4]
        date = tr[7]
        trailname = tr[3]
        ogTrailname = tr[2]
        groupSize = tr[14]
        stayLen = tr[15]
        userDays = tr[16]
        dest = tr[17]
        if trState != 'state':
            tempMatch = []
            lastTrip = 0
            matchCount = 0
            tempFullMatch = []
            #get all of the origins
            with open(origin) as csvfile:
                town = csv.reader(csvfile, delimiter=',')
                for tn in town:
                    #town variables
                    state = tn[5]
                    town = tn[3]
                    county = tn[6]
                    polsTown = tn[4]
                    #placeholder for matched trips
                    temp = {'pkTripID': tripID,'fkOriginGroupID': tn[9],'fkOriginID':tn[10],'trailName':ogTrailname, 'matchedTrailname':trailname,'mgmtUnit': mgmtUnit,'date': date,
                            'state':state,'county': county, 'town':town,'originalTown':ogTown,'NYmatchedTown':trTown,'politicalEntity':polsTown,'group_size':groupSize,
                            'stay_length':stayLen,'user_days':userDays,'destination':dest}
                    #check if town, state, and county match. if yes then add to match count and add to full match
                    #townname = 3, state = 5, countyname = 6, originID = 7
                    ## if state and county and coordtown matches town name          or state and county match and original town == town name      state and county match and (originalTown or coordtown == politicalTown) write to 100% match file
                    if (trState == state and trCounty == county and trTown.lower() == town.lower()) or (trState == state and trCounty == county and ogTown.lower() == town.lower()) or (trState == state and trCounty == county and (ogTown.lower() == polsTown.lower() or trTown.lower() == polsTown.lower())):
                        tempMatch.append(temp)
                        tempFullMatch.append(temp)
                        matchCount += 1
                    #if town and state match only one record then its also match so add match count
                    elif (trState == state and (trTown.lower() == town.lower() or ogTown.lower() == town.lower())) or (trState == state and (ogTown.lower() == polsTown.lower() or trTown.lower() == polsTown.lower())):                            
                        matchCount += 1
                        tempMatch.append(temp)
                    #if town and county are in same field (the case for some NY entries), split town and county and check match
                    elif len(trTown.split())==2 and (trState == state and (trTown).split()[1] == county and (trTown).split()[0] == town):                        
                        tempMatch.append(temp)
                        matchCount += 1
                    #NYC is a special case. If town is NYC then it gets the group ID for all NYC boroughs
                    elif ((trState == state and ogTown == 'New York City') and tn[9] == '2434'):
                        temp['town'] = 'New York City'
                        tempMatch.append(temp)
                        matchCount += 1
            #if match count is one then add to match list            
            if matchCount == 1:
                matchedEntries.append(tempMatch[0])
            #if match count is 2 check for duplicates or matches in other fields
            elif matchCount == 2:
                #if there is only one full match use that one
                if len(tempFullMatch) == 1:
                    matchedEntries.append(tempFullMatch[0])
                #if there are two identical full matches use only one
                elif len(tempFullMatch) == 2 and (tempMatch[0]['county'] == tempMatch[1]['county'] and tempMatch[0]['town'] == tempMatch[1]['town'] and tempMatch[0]['fkOriginGroupID'] == tempMatch[1]['fkOriginGroupID']):
                    matchedEntries.append(tempMatch[0])
                #if they are are two matches that have the same county and town check the political identiy
                elif (tempMatch[0]['county'] == tempMatch[1]['county'] and tempMatch[0]['town'] == tempMatch[1]['town'] and tempMatch[0]['fkOriginGroupID'] == tempMatch[1]['fkOriginGroupID']):
                    if (tempMatch[0]['originalTown'] == tempMatch[0]['politicalEntity'] or tempMatch[0]['town'] == tempMatch[0]['politicalEntity']):
                        matchedEntries.append(tempMatch[0])
                    else:
                        matchedEntries.append(tempMatch[1])
                #otherwise it goes into the unmatched list with the number of matches
                else:
                    tr.append(matchCount)
                    humanCheck.append(tr)
            #NYC as special case                            
            else:
                if (len (tempMatch) != 0 and (tempMatch[0]['state'] == 'NY' and tempMatch[0]['originalTown'] == 'New York City')):
                    matchedEntries.append(tempMatch[0])
                else:
                    tr.append(matchCount)
                    humanCheck.append(tr)

            matchCount = 0
            tempMatch = []
            tripID+=1
        
print ('Trip table sorted')

#write matched trips to a file        
with open (matchedFile, 'w', newline = '') as csvMatched:
    fieldnames = ['pkTripID','fkOriginGroupID','fkOriginID','trailName','matchedTrailname', 'mgmtUnit', 'date', 'state', 'town','originalTown','NYmatchedTown', 'politicalEntity','county', 'group_size','stay_length', 'user_days','destination' ]
    matchwriter = csv.DictWriter(csvMatched, fieldnames = fieldnames)
    matchwriter.writeheader()
    for m in matchedEntries:
        matchwriter.writerow(m)
    print ('matched files written to: '+matchedFile)

#write unmatched trips to a csv so they can be checked
with open (checkFile, 'w', newline = '') as csvCheck:
    fieldnames = ["x" ,"uid" ,"name" ,"unitmgmt" ,"decoffice" ,"page" ,"date" ,"month.number" ,"state" ,"city.original" ,"coordcity" ,"nycity.matched" ,"nycounty.matched" ,"group.size" ,"length.of.stay" ,"user.days" ,"destination" ,"matchCount"]
    checkwriter = csv.DictWriter(csvCheck, fieldnames = fieldnames)
    checkwriter.writeheader()
    for h in humanCheck:
        checkwriter.writerow({"x": h[0],"uid": h[1],"name": h[2],"unitmgmt": h[3],"decoffice": h[4],"page": h[5],"date": h[6],"month.number": h[7],"state": h[8],"city.original": h[9],"coordcity": h[10],"nycity.matched": h[11],"nycounty.matched": h[12],"group.size": h[13],"length.of.stay": h[14],"user.days": h[15],"destination": h[16],"matchCount": h[17]})
    print ('files to be rechecked written to: '+checkFile)
