##This script combines the iMap, iNaturalist and EDDmap invasive species data into a single dataset.
##It removes duplicate records and filters the results based on a csv file with a list of species of interest.
##it also assigns and origin ID to each invasive entry to match with the town dataset used for the origin ID for the trip data
##Written by Emma Tait for FEMC NY Rec Project
##Last Updated: 25 November 2019
import arcpy
import os
import csv
import argparse
import glob
import logging
import os
import re
import time
import warnings

#scratch database path
path = r'I:\WorkingProjects\NY_RecData\InvasivePlantData\scratch.gdb'
folder = r'I:\WorkingProjects\NY_RecData\SpatialData\NY_RecData.gdb'
## list of invasive plants for the region that you are interested in. 
invasivesList = r'I:\WorkingProjects\NY_RecData\InvasivePlantData\tblInvasivePlants_Copy.csv'
arcpy.env.workspace = path
arcpy.env.overwriteOutput = True
origins = folder+'\\tblOrigins'
iMap = folder+'\\iMap_9states'
eddMap = folder+'\\EDDmap_9states'
iNat = folder+'\\iNat_9states'
spatialRef = arcpy.SpatialReference('NAD 1983')

logging.basicConfig(
    level=logging.INFO,
    format=' %(asctime)s - %(levelname)s - %(message)s',
    datefmt='%I:%M:%S %p')
logger = logging.getLogger()
logger.info('initializing')


###do selection of duplicates. go through and check important info: if they are the same then append one, if they are different, append both.
###then inverse the selection to get non duplicates for each and then check those against the list of species and add the necessary ones...

eddDups = arcpy.SelectLayerByLocation_management(arcpy.MakeFeatureLayer_management(eddMap),'ARE_IDENTICAL_TO',iNat)
iNatDups = arcpy.SelectLayerByLocation_management(arcpy.MakeFeatureLayer_management(iNat),'ARE_IDENTICAL_TO',eddMap)

eddRows = arcpy.UpdateCursor(eddDups,['COMNAME','SCINAME','OBSDATE','LATITUDE','LONGITUDE', 'OBJECTID_1'])
inatRows = arcpy.UpdateCursor(iNatDups, ['species', 'verbatimScientificName', 'decimalLatitude','decimalLongitude','day','month','year','OBJECTID'])
eddInclude = []
inatInclude = []
#make sure that the points with identical geometry have identical attributes
#keep everything from EDDmap. if they don't match than keep the record from inat as well
for edd in eddRows:
    for inat in inatRows:
        if edd.SCINAME != inat.verbatimScientificName and edd.OBSDATE != inat.month+'/'+inat.day+'/'+inat.year:
            inatInclude(inat.OBJECTID)
inat = arcpy.SelectLayerByLocation_management(arcpy.MakeFeatureLayer_management(iNat),'', '','','SWITCH_SELECTION')
inatList = str(inatInclude)[1:len(str(inatInclude))-1]
if len(inatInclude) != 0:
    inat = arcpy.SelectLayerByAttribute_management(arcpy.MakeFeatureLayer_management(iNat),'ADD_TO_SELECTION','"OBJECTID" IN ('+inatList+')')
logger.info('duplicates indentified and removed from selection')
mergefields = ['common_name', 'COMNAME','scientific_name','SCINAME','scientificName','verbatimScientificName', 'OBSDATE','observation_date','day_','month_','year_','NOTES', 'State','jurisdiction','stateProvince','COMMON_NAME_PRIMARY','COMMON_NAME2','SCIENTIFIC_NAME_PRIMARY','SCIENTIFIC_NAME2','OBSERVATION_DATE', 'STATE','fkOriginGroupID','pkOriginID'] 
##function to only keep the fields that are necessary from the joins of the three shapefiles
def refineFieldmap (tables,mergefields,changeId):
    fieldmappings = arcpy.FieldMappings()
    for t in tables:
        fieldmappings.addTable(t)
    for f in fieldmappings:
         n = (f.outputField).name 
         try:
             mergefields.index(n)
         except ValueError:
             fieldmappings.removeFieldMap(fieldmappings.findFieldMapIndex(n))
    if changeId == True:
        originsindex = fieldmappings.findFieldMapIndex('pkOriginID')
        fieldmap = fieldmappings.getFieldMap(originsindex)
        field = fieldmap.outputField
        field.name = 'fkOriginID'
        fieldmap.outputField = field
        fieldmappings.replaceFieldMap(originsindex,fieldmap)
    return fieldmappings
merge = arcpy.Merge_management([eddMap, inat, iMap], path+'\\invasivesMerge_new',refineFieldmap([eddMap,inat,iMap],mergefields,False))
logger.info('invasives table merge complete')

merge = path+'\\invasivesMerge_new'
#go through plants list and get all scientific and all common names as two lists
plantcsv = open(invasivesList)
plants = csv.reader(plantcsv, delimiter=',')
commonnames = []
scinames = []
for plant in plants:
    name = plant[1].split('/') 
    sci = plant[2].split('/')
    for n in name:
        n = n.replace("'","**")
        commonnames.append(n.rstrip())           
    for s in sci:
        s = s.replace("'","**")
        scinames.append(s.rstrip())

#make a selection for if common names are in commonname and sci names are in scinames then copy the selection to new file (should be smaller and less rows to go through...)
com = str((str(commonnames)[1:len(str(commonnames))-1]))
sci = (str(scinames)[1:len(str(scinames))-1])

expression  = 'LOWER("COMNAME") IN ('+com+') OR LOWER("SCINAME") IN ('+sci+') OR LOWER("common_name") IN ('+com+') OR LOWER("scientific_name") IN ('+sci+') OR LOWER("scientificName") IN ('+sci+') OR LOWER("verbatimScientificName") IN ('+sci+')'
expression = expression.replace("**","''")
merge = arcpy.SelectLayerByAttribute_management(arcpy.MakeFeatureLayer_management(merge), 'NEW_SELECTION',expression)
merge = arcpy.CopyFeatures_management(merge, path+'\\selectedInvasives')
logger.info('selection made based on whether common name and scientific name fields are in plants list')

merge = arcpy.SpatialJoin_analysis(merge, origins, path+'\\selectedInvasives_join','','',refineFieldmap([merge,origins],mergefields,True))
mergefields[mergefields.index('pkOriginID')] =  'fkOriginID'
logger.info('join with tblOrigin complete')

addfields = ['COMMON_NAME_PRIMARY','COMMON_NAME1','SCIENTIFIC_NAME_PRIMARY','SCIENTIFIC_NAME1','DATE', 'STATE']
##add additional fields for the other common and scientific names of the species
for field in addfields:
    arcpy.AddField_management(merge,field,"TEXT",255)
logger.info('new basic fields added')
plantcsv = open(invasivesList)
plants = csv.reader(plantcsv, delimiter=',')
numCommon = 0
numScientific = 0
for plant in plants:
     name = plant[1].split('/')
     sci = plant[2].split('/')
     if len(name) > numCommon:
          numCommon = len(name)
     if len(sci)>numScientific:
          numScientific = len(sci)
for i in range(2,numCommon):
     arcpy.AddField_management(merge,'COMMON_NAME'+str(i),"TEXT",250)
     mergefields.append('COMMON_NAME'+str(i))
     logger.info('COMMON_NAME'+str(i) +' added')
for i in range(2,numScientific):
     arcpy.AddField_management(merge,'SCIENTIFIC_NAME'+str(i),"TEXT",250)
     mergefields.append('SCIENTIFIC_NAME'+str(i))
     logger.info('SCIENTIFIC_NAME'+str(i) +' added')

statedict = {'Vermont':'VT', 'New York':'NY', 'Connecticut':'CT','Rhode Island':'RI','New Hampshire': 'NH','Maine':'ME', 'Pennsylvania': 'PA', 'New Jersey':'NJ', 'Massachusetts':'MA'}
logger.info('starting row update')
merge = arcpy.CopyFeatures_management(path+'\\selectedInvasives_join', path+'\\selectedInvasivesEditing')
merge = path+'\\selectedInvasivesEditing'
###THIS GOES THROUGH ONLY THE CSV AND THEN CREATES SELECTIONS BASED ON WHETHER THE COLUMNS ARE IN THE CSV AND THEN GROUP UPDATES THE SELECTED FEATURES.
matchCount = 0
commonnames = []
scientificnames = []
merge = arcpy.MakeFeatureLayer_management(merge)
with open(invasivesList) as plantcsv:
    plants = csv.reader(plantcsv, delimiter=',')
    for plant in plants:
        plantId = plant[0]
        commonnames = [((n.lower()).rstrip()).replace("'","**") for n in plant[1].split('/')]
        scientificnames = [((s.lower()).rstrip()).replace("'","**") for s in plant[2].split('/')]
        com = str((str(commonnames)[1:len(str(commonnames))-1]))
        sci = (str(scientificnames)[1:len(str(scientificnames))-1])
        
        expression = '(LOWER("COMNAME") IN ('+com+') AND LOWER("SCINAME") IN ('+sci+')) OR (LOWER("common_name") IN ('+com+') AND LOWER("scientific_name") IN ('+sci+')) OR (LOWER("scientificName") IN ('+sci+') OR LOWER("verbatimScientificName") IN ('+sci+'))'
        expression = expression.replace("**","''")

        selection = arcpy.SelectLayerByAttribute_management(merge,'NEW_SELECTION', expression)
        if arcpy.GetCount_management(selection) != 0:
            for c in range(0,len(commonnames)):
                name = commonnames[c].replace("**","''")
                if c == 0:
                    arcpy.CalculateField_management(selection, 'COMMON_NAME_PRIMARY', '"'+name+'"')
                else:
                    fieldName = 'COMMON_NAME'+str(c)
                    arcpy.CalculateField_management(selection, fieldName, '"'+name+'"')                   
            for s in range(0,len(scientificnames)):
                sciname = scientificnames[s].replace("**","''")
                if s == 0:
                    arcpy.CalculateField_management(selection, 'SCIENTIFIC_NAME_PRIMARY', '"'+sciname+'"')
                else:
                    fieldName = 'SCIENTIFIC_NAME'+str(s)
                    arcpy.CalculateField_management(selection, fieldName, '"'+sciname+'"')
        logger.info('plant id: '+str(plantId)+' - common names: '+str(commonnames)+' processed')


matched = arcpy.SelectLayerByAttribute_management(merge,'NEW_SELECTION','"COMMON_NAME_PRIMARY" IS NOT NULL AND "fkOriginID" IS NOT NULL')
matched = arcpy.CopyFeatures_management(matched, path+'\\matchedInvasives')

##makes correct dates based on the different date formats in each file
eddCode = """
def makeEddDate (datefield):
    monthdict = {'Jan':'01', 'Feb':'02', 'Mar':'03','Apr':'04','May':'05', 'Jun':'06', 'Jul':'07','Aug':'08','Sep':'09','Oct':'10','Nov':'11','Dec':'12'}
    date = str(datefield).split()
    if len(date) == 3 and len(date[0]) == 2 and len(date[1]) == 3 and len(date[2]) == 4:
        date = date[2]+'-'+monthdict[date[1]]+'-'+date[0]
    else:
        date = None
    return date"""
imapCode = """
def makeImapDate (datefield):
    date = str(datefield).split()[0]
    return date"""
inatCode = """
def makeInatDate (day, month, year):
    date = row.year_ +'-'+row.month_+'-'+row.day_
    return date
    """
stateCode="""
def makeState(field):
    state = statedict[field]
    return state"""
##adds date and state fields
selection = arcpy.SelectLayerByAttribute_management(arcpy.MakeFeatureLayer_management(matched), 'NEW_SELECTION','"OBSDATE" IS NOT NULL')
arcpy.CalculateField_management(selection, 'DATE',"makeEddDate(!OBSDATE!)",'PYTHON3',eddCode)
arcpy.CalculateField_management(selection,'STATE', '"State"')
selection = arcpy.SelectLayerByAttribute_management(arcpy.MakeFeatureLayer_management(matched), 'NEW_SELECTION','"observation_date" IS NOT NULL')
arcpy.CalculateField_management(selection, 'DATE',"makeImapDate(!observation_date!)",'PYTHON3',imapCode)
arcpy.CalculateField_management(selection, 'STATE',"makeState(!jurisdiction!)",'PYTHON3',stateCode)
selection = arcpy.SelectLayerByAttribute_management(arcpy.MakeFeatureLayer_management(matched), 'NEW_SELECTION','"day_" IS NOT NULL')
arcpy.CalculateField_management(selection,'DATE', "makeInatDate(!day!,!month!,!year!)",'PYTHON3',inatCode)
arcpy.CalculateField_management(selection, 'STATE',"makeState(!stateProvince!)",'PYTHON3',stateCode)

#drops unecessary fields                               
dropfields = ['common_name', 'COMNAME','scientific_name','SCINAME','scientificName','verbatimScientificName', 'OBSDATE','observation_date','day_','month_','year_','NOTES', 'State','jurisdiction','stateProvince']

merge = arcpy.DeleteField_management(matched,dropfields)     
logger.info('deleting non matched rows')
invasives = arcpy.Copy_management(merge,folder+'\\tblInvasives')

invasives = folder+'\\tblInvasives'
logger.info('writing to csv')

fields = arcpy.ListFields(invasives)
fieldnames = [field.name for field in fields]
fieldnames.remove('Shape')
print (fieldnames)
##final csv of selected invasives with origin id
with open ('I:\\WorkingProjects\\NY_RecData\\SpatialData\\invasivesTables\\tblInvasives.csv', 'w', newline='') as csvfile:
    writer = csv.writer(csvfile)
    writer.writerow(fieldnames)
    rows = arcpy.SearchCursor(invasives, fieldnames)
    for row in rows:
        writer.writerow([row.OBJECTID, row.Join_Count, row.TARGET_FID, row.fkOriginGroupID, row.fkOriginID, row.COMMON_NAME_PRIMARY, row.COMMON_NAME1, row.SCIENTIFIC_NAME_PRIMARY, row.SCIENTIFIC_NAME1, row.DATE, row.COMMON_NAME2, row.COMMON_NAME3, row.COMMON_NAME4, row.COMMON_NAME5, row.COMMON_NAME6, row.COMMON_NAME7, row.COMMON_NAME8, row.SCIENTIFIC_NAME2, row.SCIENTIFIC_NAME3, row.SCIENTIFIC_NAME4, row.SCIENTIFIC_NAME5, row.SCIENTIFIC_NAME6, row.SCIENTIFIC_NAME7, row.SCIENTIFIC_NAME8, row.SCIENTIFIC_NAME9, row.SCIENTIFIC_NAME10, row.SCIENTIFIC_NAME11, row.SCIENTIFIC_NAME12, row.SCIENTIFIC_NAME13])
    logger.info('csv created')
         
logger.info('done')

               
               
  
