##This script caculates the final invasive species risk score for trailheads in the Adirondacks
##By Emma Tait for the Forest Ecosystem Monitoring Cooperative (FEMC) NY Rec Data project
##Last Updated: 26 March 2020

import arcpy
##import os
import csv
import argparse
##import glob
import logging
import os
##import re
import time
##import warnings
import math
#input and output files
originFile = r'I:\WorkingProjects\NY_RecData\FinalTables\tblOrigins.csv'
destFile = r'I:\WorkingProjects\NY_RecData\FinalTables\tblDestinations.csv'
tripFile = r'I:\WorkingProjects\NY_RecData\FinalTables\tblTrips.csv'
invasivesFile = r'I:\WorkingProjects\NY_RecData\FinalTables\tblInvasives.csv'
trailScoreFile = r'I:\WorkingProjects\NY_RecData\FinalTables\finalScores.csv'
##arcpy.env.workspace = path
logging.basicConfig(
    level=logging.INFO,
    format=' %(asctime)s - %(levelname)s - %(message)s',
    datefmt='%I:%M:%S %p')
logger = logging.getLogger()
logger.info('initializing')
#go through each trailhead and get the towns people came from and the invasives found in that town
trailList = {}
logScores = []
with open (destFile) as destcsv:
    dest = csv.reader(destcsv, delimiter=',')    
    for trail in dest:
        destID = trail[7]
        trailname = trail[8]
        trailTripOrigins = {}
        trailSpecies = {}
        logger.info(trailname)
        with open (tripFile) as tripcsv:
            trips = csv.reader(tripcsv, delimiter=',')
            #Gets the trips associated with the current trailhead and then from the trips gets the origin Ids and number of people from that town
            #associated with the tailhead
            for trip in trips:
                tripID = trip[0]
                tripOgId = trip[2]
                tripOgGrpId = trip[1]
                tripDestId = trip[3]
                tripSize = trip[15]
                #IF TRIP SIZE IS BLANK... CURRENTLY ASSUMING THAT THERE WAS AT LEAST 1 PERSON
                if tripSize == '':
                    tripSize = 1
                if tripDestId == destID:
                    if tripOgGrpId in trailTripOrigins:
                        trailTripOrigins[tripOgGrpId] += int(tripSize)
                    else: 
                        trailTripOrigins[tripOgGrpId] = int(tripSize)
        #goes through list of origin Ids and gets the invasives found in that town and sums the number of observations by species
        #then multiplies the total observations per species by the town's total number of people who went to the current trailhead.
        for origin in trailTripOrigins:
            originsSpList = {}
            pplCt = trailTripOrigins[origin]
            with open (invasivesFile) as invasivescsv:
                invasives = csv.reader(invasivescsv, delimiter=',')  
                for invasive in invasives:
                    com = invasive[5]
                    sci = invasive[7]
                    ogId = invasive[3]
                    if ogId == origin:
                        if com in originsSpList:
                            originsSpList[com]['count'] += 1
                        else:
                            originsSpList[com]= {'sci':sci, 'count': 1}
            #calculate the score using the count and the people count
            for sp in originsSpList:
                score = originsSpList[sp]['count']*pplCt
                if sp in trailSpecies:
                    trailSpecies[sp]['sum']+=score
                else:
                    trailSpecies[sp] = {'sci': originsSpList[sp]['sci'], 'sum': score}                    
            trailTripOrigins[origin] = originsSpList
            logger.info('origin: ' +origin+' processed')
        for sp in trailSpecies:
            logScore = math.log10(trailSpecies[sp]['sum'])
            trailSpecies[sp]['logScore'] = logScore
            logScores.append(logScore)
        logger.info(trailSpecies)
        trailList[trailname] = trailSpecies
logMin = min(logScores)
logMax = max(logScores)
#write final scores to a csv
with open (trailScoreFile, 'w', newline = '')as trailscorecsv:
    fieldnames = ['TrailName','SpeciesCommonName','SpeciesScientificName','SpeciesLogScore','SpeciesNormalizedScore']
    scorewriter = csv.DictWriter(trailscorecsv, fieldnames = fieldnames)
    scorewriter.writeheader()
    for trail in trailList:
        for sp in trailList[trail]:
            normScore = (trailList[trail][sp]['logScore']-logMin)/(logMax-logMin)
            trailList[trail][sp]['normalizedScore'] = normScore
            scorewriter.writerow({'TrailName':trail,'SpeciesCommonName':sp,'SpeciesScientificName':trailList[trail][sp]['sci'],'SpeciesLogScore': trailList[trail][sp]['logScore'],'SpeciesNormalizedScore':normScore})
logger.info('script complete. Scores written to: '+trailScoreFile)
            
        
